==> GFS-4.1.0.tar.gz: GFS-4.1.0/src/stomith/agents/meatware/wait_meat.c <==
/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland <teigland@sistina.com>
**
**    Module name:
**      wait_meat.c
**
**    Description:  The meatware stomith agent is two pieces.  This half
**    (wait_meatware) is started by stomithd and simply waits to hear from 
**    the second half (do_meatware).  This half is also in charge of
**    notifying a user somehow that they need to reset a machine.
**    do_meatware is run by a human after resetting the victim.
**
*******************************************************************************
******************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <syslog.h>



char *pname = "wait_meatware";

char args[256];
char *ipaddr = NULL;


void print_usage(void)
{
  printf("Usage:\n");

  printf("wait_meatware -s <ipaddr>\n\n"
	 "    -s   IP address of machine to stomith\n\n");
}


void get_options(int argc, char **argv)
{
  int c;
  char *last, *tok;
  
  if (argc > 1)
  {        
    while ((c = getopt(argc, argv, "hs:")) != -1)
    {
      switch(c)
      {
      case 'h':
	print_usage();
	exit(0);
	
      case 's':
	ipaddr = optarg;
	break;
      }
    }
  }
  
  else
  {
    memset(args, 0, 256);

    if (read(0, args, 256) < 0)
    {
      printf("STOMITH:failed:%s no input\n", pname);
      exit(0);
    }
    *(args + strlen(args) - 1) = '\0';

    for (last = "", tok = strtok(args, " "); tok; tok = strtok(NULL, " "))
    {
      if (!strncmp(last, "-s", 2))
        ipaddr = tok;

      else
      {
        if (!strncmp(tok, "-h", 2))
        {
          print_usage();
          exit(0);
        }
        
        last = tok;
        continue;
      }

      last = "";
    }
  }
}



int main(int argc, char **argv)
{
  int error, fd;
  char line[256], mw[50], result[50], ip[50];


  get_options(argc, argv);


  if (!ipaddr)
  {
    printf("STOMITH:failed:%s no IP addr\n", pname);
    exit(-1);
  }

  umask(0);
  unlink("/tmp/meatware.fifo");
  error = mkfifo("/tmp/meatware.fifo", (S_IRUSR | S_IWUSR));
  if (error)
  {
    printf("STOMITH:failed:%s mkfifo error\n", pname);
    exit(0);
  }

  openlog("wait_meatware", 0, LOG_DAEMON);

  syslog(LOG_CRIT, "Node %s requires hard reset.  Run do_meatware after "
	 "power cycling the machine.\n", ipaddr);

  
  fd = open("/tmp/meatware.fifo", O_RDONLY);
  if (fd < 0)
  {
    printf("STOMITH:failed:%s open error\n", pname);
    exit(0);
  }

  memset(line, 0, 256);
  error = read(fd, line, 256);
  if (error < 0)
  {
    printf("STOMITH:failed:%s read error\n", pname);
    exit(0);
  }
  
  memset(mw, 0, 50);
  memset(result, 0, 50);
  memset(ip, 0, 50);

  sscanf(line, "%s %s %s", mw, result, ip);

  if (strncmp(mw, "meatware", 8) || strncmp(result, "reply", 5) ||
      strncmp(ip, ipaddr, strlen(ip)))
    printf("STOMITH:failed:%s - %s\n", pname, line);
  else
    printf("STOMITH:success:%s - %s\n", pname, line);

  unlink("/tmp/meatware.fifo");

  return(0);
}

==> GFS-4.1.1.tar.gz: GFS-4.1.1/src/stomith/agents/meatware/wait_meat.c <==
/******************************************************************************
*******************************************************************************
**
**    Copyright 2000-2001 Sistina Software, Inc.
**
**    This is free software released under the GNU General Public License.
**    There is no warranty for this software.  See the file COPYING for
**    details.
**
**    See the file CONTRIBUTORS for a list of contributors.
**
**    This file is maintained by:
**      David C. Teigland <teigland@sistina.com>
**
**    Module name:
**      wait_meat.c
**
**    Description:  The meatware stomith agent is two pieces.  This half
**    (wait_meatware) is started by stomithd and simply waits to hear from 
**    the second half (do_meatware).  This half is also in charge of
**    notifying a user somehow that they need to reset a machine.
**    do_meatware is run by a human after resetting the victim.
**
*******************************************************************************
******************************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <syslog.h>



char *pname = "wait_meatware";

char args[256];
char *ipaddr = NULL;


void print_usage(void)
{
  printf("Usage:\n");

  printf("wait_meatware -s <ipaddr>\n\n"
	 "    -s   IP address of machine to stomith\n\n");
}


void get_options(int argc, char **argv)
{
  int c;
  char *last, *tok;
  
  if (argc > 1)
  {        
    while ((c = getopt(argc, argv, "hs:")) != -1)
    {
      switch(c)
      {
      case 'h':
	print_usage();
	exit(0);
	
      case 's':
	ipaddr = optarg;
	break;
      }
    }
  }
  
  else
  {
    memset(args, 0, 256);

    if (read(0, args, 256) < 0)
    {
      printf("STOMITH:failed:%s no input\n", pname);
      exit(0);
    }
    *(args + strlen(args) - 1) = '\0';

    for (last = "", tok = strtok(args, " "); tok; tok = strtok(NULL, " "))
    {
      if (!strncmp(last, "-s", 2))
        ipaddr = tok;

      else
      {
        if (!strncmp(tok, "-h", 2))
        {
          print_usage();
          exit(0);
        }
        
        last = tok;
        continue;
      }

      last = "";
    }
  }
}



int main(int argc, char **argv)
{
  int error, fd;
  char line[256], mw[50], result[50], ip[50];


  get_options(argc, argv);


  if (!ipaddr)
  {
    printf("STOMITH:failed:%s no IP addr\n", pname);
    exit(-1);
  }

  umask(0);
  unlink("/tmp/meatware.fifo");
  error = mkfifo("/tmp/meatware.fifo", (S_IRUSR | S_IWUSR));
  if (error)
  {
    printf("STOMITH:failed:%s mkfifo error\n", pname);
    exit(0);
  }

  openlog("wait_meatware", 0, LOG_DAEMON);

  syslog(LOG_CRIT, "Node %s requires hard reset.  Run do_manual "
                   "(or do_meatware) after power cycling the machine.\n", 
                   ipaddr);

  
  fd = open("/tmp/meatware.fifo", O_RDONLY);
  if (fd < 0)
  {
    printf("STOMITH:failed:%s open error\n", pname);
    exit(0);
  }

  memset(line, 0, 256);
  error = read(fd, line, 256);
  if (error < 0)
  {
    printf("STOMITH:failed:%s read error\n", pname);
    exit(0);
  }
  
  memset(mw, 0, 50);
  memset(result, 0, 50);
  memset(ip, 0, 50);

  sscanf(line, "%s %s %s", mw, result, ip);

  if (strncmp(mw, "meatware", 8) || strncmp(result, "reply", 5) ||
      strncmp(ip, ipaddr, strlen(ip)))
    printf("STOMITH:failed:%s - %s\n", pname, line);
  else
    printf("STOMITH:success:%s - %s\n", pname, line);

  unlink("/tmp/meatware.fifo");

  return(0);
}
